#pragma once
#ifndef DSY_TERRARIUM_BSP_H
#define DSY_TERRARIUM_BSP_H

#include "daisy_seed.h"

namespace daisy
{
/**
    @brief Class that handles initializing all of the hardware specific to the Daisy Terrarium from PedalPCB. \n 
    Helper funtions are also in place to provide easy access to built-in controls and peripherals.
    @author M. Anhalt (based on daisy_pod.cpp by Stephen Hensley)
    @date February 2024
    @ingroup boards
*/
class Terrarium
{
  public:
    /** Switches */
    enum Sw
    {
        FSW_1,    /** & */
        FSW_2,    /** & */
        TOGGLE_1,    /** & */
        TOGGLE_2,    /** & */
        TOGGLE_3,    /** & */
        TOGGLE_4,    /** & */
        SWITCHES_LAST, /** &  */
    };

    /** Knobs */
    enum Knob
    {
        KNOB_1,    /** &  */
        KNOB_2,    /** & */
        KNOB_3,    /** &  */
        KNOB_4,    /** & */
        KNOB_5,    /** &  */
        KNOB_6,    /** & */
        KNOB_LAST, /** & */
    };

    Terrarium() {}
    ~Terrarium() {}

    /** Init related stuff. */
    void Init(bool boost = false);

    /** Wait for a bit
    \param del Time to wait in ms.
    */
    void DelayMs(size_t del);

    /** Starts the callback
    \param cb Interleaved callback function
    */
    void StartAudio(AudioHandle::InterleavingAudioCallback cb);

    /** Starts the callback
    \param cb multichannel callback function
    */
    void StartAudio(AudioHandle::AudioCallback cb);

    /**
       Switch callback functions
       \param cb New interleaved callback function.
    */
    void ChangeAudioCallback(AudioHandle::InterleavingAudioCallback cb);

    /**
       Switch callback functions
       \param cb New multichannel callback function.
    */
    void ChangeAudioCallback(AudioHandle::AudioCallback cb);

    /** Stops the audio if it is running. */
    void StopAudio();

    /** Updates the Audio Sample Rate, and reinitializes.
     ** Audio must be stopped for this to work.
     */
    void SetAudioSampleRate(SaiHandle::Config::SampleRate samplerate);

    /** Returns the audio sample rate in Hz as a floating point number.
     */
    float AudioSampleRate();

    /** Sets the number of samples processed per channel by the audio callback.
     */
    void SetAudioBlockSize(size_t blocksize);

    /** Returns the number of samples per channel in a block of audio. */
    size_t AudioBlockSize();

    /** Returns the rate in Hz that the Audio callback is called */
    float AudioCallbackRate();

    /**
       Start analog to digital conversion.
     */
    void StartAdc();

    /** Stops Transfering data from the ADC */
    void StopAdc();

    /** Call at same rate as analog reads for smooth reading.*/
    void ProcessAnalogControls();

    /** Process Analog and Digital Controls */
    inline void ProcessAllControls()
    {
        ProcessAnalogControls();
    }

    /** & */
    float GetKnobValue(Knob k);

    /** Reset Leds*/
    void ClearLeds();

    /** Update Leds to set colors*/
    void UpdateLeds();

    /** Public Members */
    DaisySeed     seed;        /**<# */
    AnalogControl knob1,       /**< & */
        knob2,                 /**< & */
        knob3,                 /**< & */
        knob4,                 /**< & */
        knob5,                 /**< & */
        knob6,                 /**< & */
        *knobs[KNOB_LAST];     /**< & */
    Switch fsw1,            /**< & */
        fsw2,               /**< & */
        toggle1,               /**< & */
        toggle2,               /**< & */
        toggle3,               /**< & */
        toggle4,               /**< & */
        *switches[SWITCHES_LAST]; /**< & */
    RgbLed led1,               /**< & */
        led2;                  /**< & */
    MidiUartHandler midi;

  private:
    void SetHidUpdateRates();
    void InitSwitches();
    void InitLeds();
    void InitKnobs();
};

} // namespace daisy
#endif
